/*
** Module   : SelfManager.java
** Abstract : A manager for the SELF system handle.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130121 Created initial version.
*/

package com.goldencode.p2j.util;

import java.util.*;
import com.goldencode.p2j.security.*;

/**
 * This class manages scoping of the SELF system handle. Before the event executes its trigger/
 * procedure/etc, the manager for that event must call {@link #pushSelf(handle)} to save the 
 * latest handle on top of the stack. After the event has finish, the manager for that event must
 * call {@link #popSelf()}. 
 */
public class SelfManager
{
   /** Context-local data variable. */
   private static ContextLocal<WorkArea> local = new ContextLocal<WorkArea>();
   
   /**
    * Get the latest self handle. If this is called outside of a event processing block, it will
    * return unknown.
    * 
    * @return   See above.
    */
   public static handle self()
   {
      WorkArea wa = local.get();
      if (wa.self.isEmpty())
      {
         return new handle();
      }
      
      return new handle(wa.self.peek());
   }
   
   /**
    * Discard the last SELF handle.
    * 
    * @return   The last SELF handle.
    */
   public static handle popSelf()
   {
      return new handle(local.get().self.pop());
   }
   
   /**
    * Push the given handle as the new SELF handle.
    * 
    * @param    h
    *           The new SELF handle reference.
    */
   public static void pushSelf(handle h)
   {
      local.get().self.push(h);
   }
   
   /**
    * Container of context-local data.
    */
   private static class WorkArea
   {
      /** Stack of SELF handles. */
      private Deque<handle> self = new ArrayDeque<handle>();
   }
}